Package: scanMiRApp
Type: Package
Title: scanMiR shiny application
Version: 1.19.0
Date: 2026-02-25
Authors@R: c(
    person("Pierre-Luc", "Germain", email="pierre-luc.germain@hest.ethz.ch", 
      role=c("cre","aut"), comment=c(ORCID="0000-0003-3418-4218")),
    person("Michael", "Soutschek", email="michael.soutschek@hest.ethz.ch", role="aut"),
    person("Fridolin", "Gross", email="fridolin.gross@hest.ethz.ch", role="ctb"))
Imports: AnnotationDbi, AnnotationFilter, AnnotationHub, BiocParallel,
        Biostrings, data.table, digest, DT, ensembldb, fst,
        GenomeInfoDb, GenomicFeatures, GenomicRanges, ggplot2,
        htmlwidgets, IRanges, Matrix, methods, plotly, rintrojs,
        rtracklayer, S4Vectors, scanMiRData, shiny, shinycssloaders,
        shinydashboard, shinyjqui, stats, utils, txdbmaker, waiter
Suggests: knitr, rmarkdown, BiocStyle, testthat (>= 3.0.0), shinytest,
        BSgenome.Hsapiens.UCSC.hg38, BSgenome.Mmusculus.UCSC.mm10,
        BSgenome.Mmusculus.UCSC.mm39, BSgenome.Rnorvegicus.UCSC.rn6
Description: 
    A shiny interface to the scanMiR package. The application enables the 
    scanning of transcripts and custom sequences for miRNA binding sites, the 
    visualization of KdModels and binding results, as well as browsing predicted
    repression data. In addition contains the IndexedFst class for fast indexed
    reading of large GenomicRanges or data.frames, and some utilities for 
    facilitating scans and identifying enriched miRNA-target pairs.
Depends: R (>= 4.0), scanMiR
License: GPL-3
VignetteBuilder: knitr
RoxygenNote: 7.2.3
biocViews: miRNA, SequenceMatching, GUI, ShinyApps
Config/testthat/edition: 3
git_url: https://git.bioconductor.org/packages/scanMiRApp
git_branch: devel
git_last_commit: 7b6d92e
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-29 00:52:26 UTC; biocbuild
Author: Pierre-Luc Germain [cre, aut] (ORCID:
    <https://orcid.org/0000-0003-3418-4218>),
  Michael Soutschek [aut],
  Fridolin Gross [ctb]
Maintainer: Pierre-Luc Germain <pierre-luc.germain@hest.ethz.ch>
