# Generated by roxygen2: do not edit by hand

export(ScanMiRAnno)
export(enrichedMirTxPairs)
export(getTranscriptSequence)
export(loadIndexedFst)
export(plotSitesOnUTR)
export(runFullScan)
export(saveIndexedFst)
export(scanMiRApp)
export(scanMiRserver)
export(scanMiRui)
exportClasses(IndexedFst)
exportClasses(ScanMiRAnno)
exportMethods("$")
exportMethods("[")
exportMethods("[[")
exportMethods(as.data.frame)
exportMethods(colnames)
exportMethods(head)
exportMethods(length)
exportMethods(lengths)
exportMethods(names)
exportMethods(ncol)
exportMethods(nrow)
exportMethods(show)
exportMethods(summary)
import(Biostrings)
import(GenomicRanges)
import(IRanges)
import(Matrix)
import(methods)
import(scanMiR)
import(shiny)
import(shinydashboard)
importFrom(AnnotationDbi,select)
importFrom(AnnotationFilter,AnnotationFilter)
importFrom(AnnotationFilter,AnnotationFilterList)
importFrom(AnnotationFilter,SeqNameFilter)
importFrom(AnnotationHub,AnnotationHub)
importFrom(AnnotationHub,query)
importFrom(BiocParallel,MulticoreParam)
importFrom(BiocParallel,SerialParam)
importFrom(BiocParallel,bpnworkers)
importFrom(Biostrings,DNAString)
importFrom(Biostrings,DNAStringSet)
importFrom(Biostrings,RNAString)
importFrom(DT,DTOutput)
importFrom(DT,datatable)
importFrom(DT,renderDT)
importFrom(GenomeInfoDb,seqlevels)
importFrom(GenomicFeatures,"seqlevels<-")
importFrom(GenomicFeatures,cds)
importFrom(GenomicFeatures,cdsBy)
importFrom(GenomicFeatures,exons)
importFrom(GenomicFeatures,exonsBy)
importFrom(GenomicFeatures,extractTranscriptSeqs)
importFrom(GenomicFeatures,threeUTRsByTranscript)
importFrom(GenomicRanges,GRanges)
importFrom(GenomicRanges,makeGRangesFromDataFrame)
importFrom(GenomicRanges,strand)
importFrom(IRanges,IRanges)
importFrom(S4Vectors,"mcols<-")
importFrom(S4Vectors,"metadata<-")
importFrom(S4Vectors,mcols)
importFrom(S4Vectors,metadata)
importFrom(data.table,rbindlist)
importFrom(digest,digest)
importFrom(ensembldb,"seqlevelsStyle<-")
importFrom(ensembldb,exonsBy)
importFrom(ensembldb,genes)
importFrom(ensembldb,getGenomeTwoBitFile)
importFrom(ensembldb,metadata)
importFrom(ensembldb,seqlevelsStyle)
importFrom(ensembldb,threeUTRsByTranscript)
importFrom(ensembldb,transcripts)
importFrom(fst,fst.metadata)
importFrom(fst,read.fst)
importFrom(fst,threads_fst)
importFrom(fst,write.fst)
importFrom(ggplot2,aes)
importFrom(ggplot2,aes_string)
importFrom(ggplot2,element_line)
importFrom(ggplot2,expand_limits)
importFrom(ggplot2,geom_hline)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,geom_rect)
importFrom(ggplot2,geom_text)
importFrom(ggplot2,geom_vline)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,labs)
importFrom(ggplot2,scale_x_continuous)
importFrom(ggplot2,scale_y_continuous)
importFrom(ggplot2,theme)
importFrom(ggplot2,theme_light)
importFrom(ggplot2,theme_minimal)
importFrom(ggplot2,xlab)
importFrom(ggplot2,xlim)
importFrom(htmlwidgets,JS)
importFrom(methods,show)
importFrom(plotly,event_data)
importFrom(plotly,event_register)
importFrom(plotly,ggplotly)
importFrom(plotly,plotlyOutput)
importFrom(plotly,renderPlotly)
importFrom(rintrojs,hintjs)
importFrom(rintrojs,introBox)
importFrom(rintrojs,introjs)
importFrom(rintrojs,introjsUI)
importFrom(rintrojs,readCallback)
importFrom(rtracklayer,TwoBitFile)
importFrom(rtracklayer,export.2bit)
importFrom(scanMiRData,getKdModels)
importFrom(shinycssloaders,withSpinner)
importFrom(shinyjqui,jqui_resizable)
importFrom(stats,pbinom)
importFrom(stats,setNames)
importFrom(txdbmaker,makeTxDbFromGRanges)
importFrom(utils,capture.output)
importFrom(utils,data)
importFrom(utils,object.size)
importFrom(utils,packageVersion)
importFrom(utils,write.csv)
importFrom(waiter,spin_1)
importFrom(waiter,use_waiter)
importFrom(waiter,waiter_hide)
importFrom(waiter,waiter_show)
importFrom(waiter,waiter_show_on_load)
importMethodsFrom(methods,show)
