% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\name{fakeTxDb}
\alias{fakeTxDb}
\title{Example 'fake' TxDb object}
\value{
a named character vector of length 1.
}
\description{
A fake transcript database used for examples.
}
