% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scanMiRui.R
\name{scanMiRui}
\alias{scanMiRui}
\title{scanMiRui}
\usage{
scanMiRui()
}
\value{
A shiny ui
}
\description{
UI for the scanMiR app.
}
\examples{
ui <- scanMiRui()
}
